@@ -7,11 +7,13 @@
 // Media Patent License 1.0 was not distributed with this source code in the
 // PATENTS file, you can obtain it at www.aomedia.org/license/patent.
 
-use crate::serialize::{Deserialize, Serialize};
-use crate::wasm_bindgen::*;
+#[cfg(feature = "serialize")]
+use serde::{Deserialize, Serialize};
 
-use num_derive::FromPrimitive;
-use num_traits::{AsPrimitive, PrimInt, Signed};
+#[cfg(all(target_arch = "wasm32", target_os = "unknown"))]
+use wasm_bindgen::prelude::*;
+
+use num_traits::{AsPrimitive, FromPrimitive, PrimInt, Signed};
 
 use std::fmt;
 use std::fmt::{Debug, Display};
@@ -108,7 +110,6 @@ pub trait Pixel:
 
     /// Converts stride in pixels to stride in bytes.
     #[inline]
-    #[allow(clippy::wrong_self_convention)]
     fn to_asm_stride(in_stride: usize) -> isize {
         (in_stride * size_of::<Self>()) as isize
     }
@@ -146,8 +147,9 @@ impl Coefficient for i32 {
 }
 
 /// Chroma subsampling format
-#[wasm_bindgen]
-#[derive(Copy, Clone, Debug, Default, PartialEq, Eq, FromPrimitive, Serialize, Deserialize)]
+#[derive(Copy, Clone, Debug, Default, PartialEq, Eq)]
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+#[cfg_attr(all(target_arch = "wasm32", target_os = "unknown"), wasm_bindgen)]
 #[repr(C)]
 pub enum ChromaSampling {
     /// Both vertically and horizontally subsampled.
@@ -161,6 +163,24 @@ pub enum ChromaSampling {
     Cs400,
 }
 
+impl FromPrimitive for ChromaSampling {
+    fn from_i64(n: i64) -> Option<Self> {
+        use ChromaSampling::*;
+
+        match n {
+            n if n == Cs420 as i64 => Some(Cs420),
+            n if n == Cs422 as i64 => Some(Cs422),
+            n if n == Cs444 as i64 => Some(Cs444),
+            n if n == Cs400 as i64 => Some(Cs400),
+            _ => None,
+        }
+    }
+
+    fn from_u64(n: u64) -> Option<Self> {
+        ChromaSampling::from_i64(n as i64)
+    }
+}
+
 impl fmt::Display for ChromaSampling {
     fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
         write!(
@@ -206,3 +226,91 @@ impl ChromaSampling {
         }
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    #[cfg(all(target_arch = "wasm32", target_os = "unknown"))]
+    use wasm_bindgen_test::*;
+
+    #[cfg(all(target_arch = "wasm32", target_os = "unknown"))]
+    wasm_bindgen_test_configure!(run_in_browser);
+
+    #[cfg_attr(all(target_arch = "wasm32", target_os = "unknown"), wasm_bindgen_test)]
+    #[test]
+    fn asm_stride() {
+        let tests = [(7, 7, 14), (12, 12, 24), (1234, 1234, 2468)];
+
+        for (in_stride, u8_bytes, u16_bytes) in tests {
+            assert_eq!(u8::to_asm_stride(in_stride), u8_bytes);
+            assert_eq!(u16::to_asm_stride(in_stride), u16_bytes);
+        }
+    }
+
+    #[cfg_attr(all(target_arch = "wasm32", target_os = "unknown"), wasm_bindgen_test)]
+    #[test]
+    fn type_enum() {
+        assert!(u8::type_enum() == PixelType::U8);
+        assert!(u16::type_enum() == PixelType::U16);
+    }
+
+    #[cfg_attr(all(target_arch = "wasm32", target_os = "unknown"), wasm_bindgen_test)]
+    #[test]
+    fn chroma_sampling_from_int() {
+        let expected = [
+            (-1, None),
+            (0, Some(ChromaSampling::Cs420)),
+            (1, Some(ChromaSampling::Cs422)),
+            (2, Some(ChromaSampling::Cs444)),
+            (3, Some(ChromaSampling::Cs400)),
+            (4, None),
+        ];
+
+        for (int, chroma_sampling) in expected {
+            let converted = ChromaSampling::from_i32(int);
+            assert_eq!(chroma_sampling, converted);
+
+            let converted_uint = ChromaSampling::from_u32(int as u32);
+            assert_eq!(chroma_sampling, converted_uint);
+        }
+    }
+
+    #[cfg_attr(all(target_arch = "wasm32", target_os = "unknown"), wasm_bindgen_test)]
+    #[test]
+    fn display_chroma_sampling() {
+        use std::fmt::Write;
+
+        let all_cs = [
+            (ChromaSampling::Cs420, "4:2:0"),
+            (ChromaSampling::Cs422, "4:2:2"),
+            (ChromaSampling::Cs444, "4:4:4"),
+            (ChromaSampling::Cs400, "Monochrome"),
+        ];
+
+        for (cs, expected) in all_cs {
+            let mut s = String::new();
+            write!(&mut s, "{cs}").expect("can display");
+            assert_eq!(&s, expected);
+        }
+    }
+
+    #[cfg_attr(all(target_arch = "wasm32", target_os = "unknown"), wasm_bindgen_test)]
+    #[test]
+    fn chroma_sampling_dimensions() {
+        let tests = [
+            ((1024, 768), ChromaSampling::Cs444, (1024, 768)),
+            ((1024, 768), ChromaSampling::Cs422, (512, 768)),
+            ((1024, 768), ChromaSampling::Cs420, (512, 384)),
+            ((1024, 768), ChromaSampling::Cs400, (0, 0)),
+            // Check odd width/height
+            ((1023, 768), ChromaSampling::Cs422, (512, 768)),
+            ((1023, 767), ChromaSampling::Cs420, (512, 384)),
+        ];
+
+        for (luma, cs, expected_chroma) in tests {
+            let chroma = cs.get_chroma_dimensions(luma.0, luma.1);
+            assert_eq!(chroma, expected_chroma);
+        }
+    }
+}
